// Bapple2 include file

#ifndef _BAPPLE2_

#define _BAPPLE2_

// Win32 stuff

#ifndef GFX_MODEX
#define GFX_MODEX	2362
#endif

// End Win32 stuff

#include "units.h"

#define BETWEEN(var, left, right)  (((var) > (left)) && ((var) < (right)))

#define GAME_STATE_EXIT			0
#define GAME_STATE_PLAY 		1
#define GAME_STATE_MAP			2
#define GAME_STATE_HELP         3
#define GAME_STATE_INITCONN		4

#define MAX_PLAYERS		64	// not
#define NEUTRAL			-1


#define TERRAIN_TILES	30

#define far
#define near

// Terrain types
#define TERRAIN_WATER	0
#define TERRAIN_LAND	1
#define TERRAIN_COAST	2
#define TERRAIN_BLOCKED	3
#define TERRAIN_ISLAND	4

// Visibility (stored in vis)
#define VIS_DARK		0
#define VIS_LIGHT		1

#define INPUT_LEFT			1
#define INPUT_RIGHT			2
#define INPUT_FORWARD		4
#define INPUT_BACK			8
#define INPUT_AFTERBURNER	16

// Rendering methods
#define PAGE_FLIP		0
#define DOUBLE_BUFFER	1

// Stuff for player movement interrupt handler
#define PLAYER_UPDATE	40

// GRAPHIC BITMAP DEFINITIONS
//#define NUM_TILES   64

// Communications stuff
#define COMM_NONE	0
#define COMM_CLIENT	1
#define COMM_SERVER	2

#define CONN_NONE	0
#define CONN_WSOCK  1
#define CONN_SERIAL	2

// Communications states for enemies
#define CS_NONEXISTENT	0
#define CS_CONNECTED	1
#define CS_INITIALIZING	2

extern void options_dialog(void);
extern DIALOG the_dialog[];

// Used for the sound initialization routines..
// so they can use argv[0] to find the directory to look for a sound.cfg
extern int __crt0_argc;
extern char **__crt0_argv;

// Macro for finding the distance between two points using floats
#define Distance(x1,y1,x2,y2) sqrt((double)(((x2 - x1) * (x2 - x1)) + ((y2 - y1) * (y2 - y1))))

// Macro for accessing a char-based linear address space with two subscripts
#define mpos(a, y, x)	(a)[((MAP_WIDTH * (y)) + (x))]

// Macro for accessing a short-int-based linear address space with two subscripts
//#define tmpos(a, y, x)	(a)[(((MAP_WIDTH * (y)) + (x)) * sizeof(short))]

// Macro for accessing a pointer-based linear address space with two subscripts
//#define tmpos(a, y, x)	(a)[(((MAP_WIDTH * (y)) + (x)) * sizeof(void *))]

/* Macro to wait for the vertical retrace */
#define waitforretrace()  while(!(inp(0x3DA)& 8))

#define PLAYER_ENEMY_COLLISION_DISTANCE 20
//#define INITIAL_BACKGROUND_MUSIC

#define fix_string(a)       a[strlen(a)-1] = '\0'

#define NUM_PLAYER_ROTATIONS    256
#define D_EAST          0
#define D_NORTHEAST     2
#define D_NORTH         4
#define D_NORTHWEST     6
#define D_WEST          8
#define D_SOUTHWEST     10
#define D_SOUTH         12
#define D_SOUTHEAST     14

#define ON                      1
#define OFF                     0

#define NUM_GUNS                6

// This had been switched to a global int in globals.cc, but I switched it back
//#define TEXT_COLOR                  160 // 123
#define TEXT_COLOR		(makecol(223, 207, 239))

#define ENEMY_HEALTH_DECREMENT  40

//#define MAX_PROJECTILES             400     // 100
#define TOASTED_PROJECTILE          -1
#define NUM_PROJECTILE_TYPES        6
// Possible team values
#define PROJECTILE_TEAM_UNIT		255
// Possible type values
#define PROJECTILE_TYPE_YELLOW  0
#define PROJECTILE_TYPE_GREEN       1
#define PROJECTILE_TYPE_RED         2
#define PROJECTILE_TYPE_MISSILE 3
#define PROJECTILE_TYPE_LASER       4
#define PROJECTILE_TYPE_SPREAD      5

#define MAX_EXPLOSIONS              50
#define EXPLOSION_NONEXISTENT       -1
#define SMALL_EXPLOSION             0
#define BIG_EXPLOSION               1
#define EXPLOSION_RATE              1
#define NUM_EXPLOSION_FRAMES        7

#define PLAYER_WIDTH                28
#define PLAYER_HEIGHT               31

#define ENEMY_WIDTH                 32
#define ENEMY_HEIGHT                    28

#define PROJECTILE_PLAYER_WIDTH 6
#define PROJECTILE_PLAYER_HEIGHT    6

#define PROJECTILE_ENEMY_WIDTH  6
#define PROJECTILE_ENEMY_HEIGHT 6

#define STRAIGHT                        0
#define UP                              1
#define DOWN                            2
#define LEFT                            3
#define RIGHT                           4

#define YES                             1
#define NO                              0

typedef struct {
    unsigned long last_shot_time;
    char shot_rate;
	char input;					// Bitfield for input states (accel and turning)
    fixed speed;
    fixed maxspeed;
	signed char shiptype;		// # of ship that this player is using
    int x;
    int y;
    signed int xoff;
    signed int yoff;
	float fxoff;
	float fyoff;
    int health;
	int fuel;
	short income;				// Player income (how much money) per income tick
	int money;					// Amount of money that the player currently has
    int direction;				// Visible direction
    int targetdir;				// Direction of motion
	char hidden;				// Hide the player from view?  (like when they're dead)
	unit_t *unit_carried;		// Unit being carried by this player
	struct MBASE_NODE *mbase;	// Main-bases that this player owns

	// Chopper animation stuff
	char rotorframe;
	unsigned long rotortime;

	// Comm stuff
	char name[16];				// Name
    unsigned char id;			// ID # (assigned by the server)
	char comm_state;			// Communications state (connected, initializing, etc)
	char addr[80];				// Network address
	unsigned long latency;		// Last time (ms_time) that we heard anything from this player
	unsigned char keepalives;	// Number of keepalives sent to the client so far
	short int kills;			// Number of kills
	short int deaths;			// Number of deaths
	unsigned char lastattacker;	// The ID # of the last person who hurt this player
	unsigned long pingtime;		// Last time we began a ping to this player
    int sentpackets;			// # of packets sent to this player
	int recvpackets;			// # of packets received from this player
	int seqnum;					// Packet sequence number as reported by this player
} player_t;

typedef struct {
    fixed speed;
    signed char type;
    unsigned char team;
    int direction;
    unsigned long dietime;
    int x, y;
    signed int xoff, yoff;
	unit_t *utarget;
	player_t *ptarget;
} projectile_struct;

typedef struct {
    long last_explosion_time;
    signed char frame;
    char numframes;
    signed char type;
    int x, y;
	signed int xoff, yoff;
} explosion_struct;

typedef struct thing_t {
	char type;
	void *ptr;
	struct thing_t *next;
} thing_t;


// PROJECTILE RELATED STUFF

#define MAX_PROJECTILES         100
#define PROJ_NONEXISTENT        -1
#define PROJ_RED                0

struct BASE_NODE {				// MINI-BASE STRUCTURE
	int x, y;					// Upper-left map coordinates
	signed char owner;			// Who owns this base?      (-1 = neutral)
    signed char dot[2][4];		// Who has partial control? (-1 = neutral)
	struct BASE_NODE *next;     // Pointer to next base
};

struct MBASE_NODE {             // MAIN-BASE STRUCTURE
	int x, y;					// Upper-left map coordinates
	signed char owner;			// Who owns this base?      (-1 = neutral)
	int health;					// Health of the structure
	struct MBASE_NODE *next;    // Pointer to next base
};

#define UA_FREE		0
#define UA_SINGLE	1
#define UA_GO		2
#define UA_WAIT		3

// The threshold, in # of tiles, between world sounds you can and cannot hear
#define HEARING_THRESHOLD	42.0

#define NUM_CHUNKS	1

#endif // #ifdef _BAPPLE2_

// Notes
// -----

// Frag/hurt protocol: (the simple explanation)
/*
 * See, when someone hurts someone else in the game, they say "Hey, I just
 * hurt you!"  Then the hurt person remembers the last person who hurt them...
 * So when they finally die, they know who to blame it on. :)  Then they'll
 * send a frag signal to the server which tells who gets the credit, and of
 * course the server will relay it around to the other clients.
 */
