// base.c: Base related functions for Bapple2.  Created on April 12th, 1998

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>

#include <allegro.h>
#include "bapple2.h"
#include "bap2dat.h"
#include "globals.h"
#include "map.h"
#include "msg.h"

void init_bases(void);
void process_bases(void);
void create_base(short x, short y, signed char owner);
void create_mbase(short x, short y, signed char owner);
void draw_base(struct BASE_NODE *base);
void draw_mbase(struct MBASE_NODE *base);
void place_bases(void);
void destroy_base(struct BASE_NODE *b);
void destroy_mbase(struct MBASE_NODE *b);

/* init_bases():
 *   Initializes the main linked list pointers to NULL.
 */
void init_bases(void)
{
	short i;

    base = NULL;
    mbase = NULL;
}

/* process_bases(): -------------------------------------------------------
 *   Processes bases.  Currently checks to see if any of them are ready to
 *   explode and be removed from the world.
 */
void process_bases(void)
{
	struct BASE_NODE *bp;
	struct MBASE_NODE *mbp;
	char done;
	int x, y;

	do
	{
		done = 1;
		for (mbp = mbase; mbp; mbp = mbp->next)
			if (mbp->health <= 0)
			{
				for (y = mbp->y; y < mbp->y + 5; y++)
					for (x = mbp->x; x < mbp->x + 5; x++)
					{
						mpos(frill, y, x) = 0;
		                create_explosion(BIG_EXPLOSION, x, y, (rand() % 16) - 8, (rand() % 16) - 8);
					}

				if ((player.mbase) && (mbp->owner == player.id))
					add_msg(MSG_COMPUTER, "Aw, your base is toast..  better luck next time!");

	            destroy_mbase(mbp);
				done = 0;
				break;
			}
	} while (!done);
}

/* create_base():
 *   Creates a base given its location and owner (or neutral).
 *   Allocates space for the new node and adds it to the appropriate list(s).
 */
void create_base(short x, short y, signed char owner)
{
	struct BASE_NODE *newnode;
	short i, j;

	// Allocate memory for a new base
	newnode = (struct BASE_NODE *) calloc(1, sizeof(struct BASE_NODE));

	newnode->x = x;
	newnode->y = y;
	newnode->owner = owner;
	for (i = 0; i < 2; i++)
		for (j = 0; j < 4; j++)
			newnode->dot[i][j] = NEUTRAL;

	// Insert node into global base list
    newnode->next = base;
	base = newnode;

	// Draw the base onto the maps (visible and terrain)
	draw_base(newnode);
}

/* create_mbase():
 *   Creates a main base given its location and owner.
 *   Allocates space for the new node and adds it to the appropriate list(s).
 */
void create_mbase(short x, short y, signed char owner)
{
	struct MBASE_NODE *newnode;
	short i, j;

	// Allocate memory for a new base
	newnode = (struct MBASE_NODE *) calloc(1, sizeof(struct MBASE_NODE));

	newnode->x = x;
	newnode->y = y;
	newnode->owner = owner;
    newnode->health = 10000;

	// Insert node into global base list
    newnode->next = mbase;
	mbase = newnode;

	// Handle individual ownership list (if not neutral)
	if (owner != NEUTRAL)
		if (owner == player.id)
			player.mbase = newnode;

	// Draw the base onto the maps (visible and terrain)
	draw_mbase(newnode);
}

/* draw_base():
 *   This takes a base node and draws it onto the tile maps.
 *   It should eventually take ownership into account! (FIXME)
 */
void draw_base(struct BASE_NODE *base)
{
	int x, y, r, tileoffset;

	// If we were passed a null pointer, bail
	if (base == NULL)
		return;

	if ((base->x < 0) || (base->x > MAP_WIDTH - 3) || (base->y < 0) || (base->y > MAP_HEIGHT - 3))
	{
		add_msg(MSG_COMPUTER, "Warning: Mini-base coordinates (%d, %d) out of range!", base->x, base->y);
		return;
	}

	// For now, we start with a fixed tile offset (yak!)
	tileoffset = 26;
	for (y = base->y; y < (base->y + 3); y++)
		for (x = base->x; x < (base->x + 3); x++)
			if (BETWEEN(x, 0, MAP_WIDTH) && BETWEEN(y, 0, MAP_HEIGHT))
			{
				mpos(frill, y, x) = tileoffset++;		// Draw to frill layer
				mpos(terrain, y, x) = TERRAIN_BLOCKED;	// Draw to terrain map
			}

	// If it's a friendly base, uncover some of the map around it
/*	if (base->owner == player.id)
	{
		r = 12;
		for (y = base->y + 1 - r; y <= base->y + 1 + r; y++)
			for (x = base->x + 1 - r; x <= base->x + 1 + r; x++)
				if (BETWEEN(x, 0, MAP_WIDTH) && BETWEEN(y, 0, MAP_HEIGHT))
					mpos(vis, y, x) = VIS_LIGHT;
		redraw_terrain_bmp = 1;
	} */
}

/* draw_mbase():
 *   This takes a main base node and draws it onto the tile maps.
 *   It should eventually take ownership into account! (FIXME)
 */
void draw_mbase(struct MBASE_NODE *base)
{
	int x, y, r, tileoffset;

	// If we were passed a null pointer, bail
	if (base == NULL)
		return;

//	add_msg(MSG_COMPUTER, "Drawing mbase at %d,%d", base->x, base->y);

	if ((base->x < 0) || (base->x > MAP_WIDTH - 3) || (base->y < 0) || (base->y > MAP_HEIGHT - 3))
	{
		add_msg(MSG_COMPUTER, "Warning: Main-base coordinates (%d, %d) out of range!", base->x, base->y);
		return;
	}

	// For now, we start with a fixed tile offset (yak!)
	tileoffset = 1;
	for (y = base->y; y < (base->y + 5); y++)
		for (x = base->x; x < (base->x + 5); x++)
		{
			mpos(frill, y, x) = tileoffset++;		// Draw to frill layer
			mpos(terrain, y, x) = TERRAIN_BLOCKED;	// Draw to terrain map
		}
	// Draw some flags
	mpos(frill, base->y - 1, base->x - 1) = 35;
	mpos(frill, base->y - 1, base->x + 5) = 36;
	mpos(frill, base->y + 5, base->x - 1) = 37;
	mpos(frill, base->y + 5, base->x + 5) = 38;

	// If it's a friendly base, uncover some of the map around it
/*	if (base->owner == player.id)
	{
		r = 24;
		for (y = base->y + 1 - r; y <= base->y + 1 + r; y++)
			for (x = base->x + 1 - r; x <= base->x + 1 + r; x++)
				if (BETWEEN(x, 0, MAP_WIDTH) && BETWEEN(y, 0, MAP_HEIGHT))
					mpos(vis, y, x) = VIS_LIGHT;
		redraw_terrain_bmp = 1;
	} */
}

/* place_bases():
 *   Throw some bases and mini-bases into the world..
 */
void place_bases(void)
{
	int x, y, good, px, py, ctr, numbases, owner;

// *** MAIN BASES ***

	numbases = 0;
	owner = 0;
	while (numbases++ < num_initial_mainbases)
	{
		good = ctr = 0;
		while (!good)
		{
			ctr++;
			if (ctr > 10000)
			{
				add_msg(MSG_COMPUTER, "I tried hard but could only create %d out of the requested %d main-bases.", numbases, num_initial_mainbases);
//				add_msg(MSG_COMPUTER, "Press a key to continue..");
//				getch();
				numbases = num_initial_mainbases;
				break;
			}
			px = rand() % MAP_WIDTH;
	        py = rand() % MAP_HEIGHT;
	        good = 1;
			// Watch out for borders of map, and look for a clearing of land
			// Revised to give the base an extra margin of free space for flags
	        for (y = py - 1; ((y < py + 7) && good); y++)
	            for (x = px - 1; ((x < px + 7) && good); x++)
	                if (px > MAP_WIDTH - 7)
	                    good = 0;
	                else if (py > MAP_HEIGHT - 7)
	                    good = 0;
					else if ((px < 7) || (py < 7))
						good = 0;
	                else if (mpos(terrain, y, x) != TERRAIN_LAND)
	                    good = 0;
			// Try avoiding close proximity to other bases (mini or main)
	        for (y = py - 5; ((y < py + 10) && good); y++)
	            for (x = px - 5; ((x < px + 10) && good); x++)
	                if (px > MAP_WIDTH - 7)
	                    continue;
	                else if (py > MAP_HEIGHT - 7)
	                    continue;
					else if ((px < 7) || (py < 7))
	                    continue;
	                else if ((mpos(map, y, x) == 42) || (mpos(map, y, x) == 59))
	                    good = 0;
		}
		if (good)
		{
			create_mbase(px, py, owner++);		// cycle through owner IDs
/*			if (numbases == 1)
		        create_mbase(px, py, player.id);	// The player gets the 1st base
			else
			if ((numbases == 2) && (comm_mode == COMM_NONE))
				create_mbase(px, py, player.id + 1);
			else
		        create_mbase(px, py, NEUTRAL);	// Just make the other ones neutral bases */
		}
	}

// *** MINI-BASES ***

	numbases = 0;
	while (numbases++ < num_initial_minibases)
	{
		good = ctr = 0;
		while (!good)
		{
			ctr++;
			if (ctr > 10000)
			{
				add_msg(MSG_COMPUTER, "I tried hard but could only create %d out of the requested %d mini-bases.", numbases, num_initial_minibases);
//				add_msg(MSG_COMPUTER, "Press a key to continue..");
//				getch();
				return;
			}
			px = rand() % MAP_WIDTH;
	        py = rand() % MAP_HEIGHT;
	        good = 1;
			// Watch out for borders of map, and look for a clearing of land
			// Revised to give the base an extra margin of free space for flags
	        for (y = py - 1; ((y < py + 5) && good); y++)
	            for (x = px - 1; ((x < px + 5) && good); x++)
	                if (px > MAP_WIDTH - 5)
	                    good = 0;
	                else if (py > MAP_HEIGHT - 5)
	                    good = 0;
					else if ((px < 5) || (py < 5))
						good = 0;
	                else if (mpos(terrain, y, x) != TERRAIN_LAND)
	                    good = 0;
			// Try avoiding close proximity to other bases (mini or main)
	        for (y = py - 3; ((y < py + 6) && good); y++)
	            for (x = px - 3; ((x < px + 6) && good); x++)
	                if (px > MAP_WIDTH - 5)
	                    continue;
	                else if (py > MAP_HEIGHT - 5)
	                    continue;
					else if ((px < 5) || (py < 5))
	                    continue;
	                else if ((mpos(map, y, x) == 42) || (mpos(map, y, x) == 59))
	                    good = 0;
	        if (!good)
				continue;
		}
		if (comm_mode == COMM_NONE)
	        create_base(px, py, player.id);
		else
	        create_base(px, py, NEUTRAL);	// Just neutral bases to start with
	}
}

/* destroy_base():
 *   Destroys a mini-base given a pointer to it.
 */
void destroy_base(struct BASE_NODE *b)
{
	struct BASE_NODE *p;

	// If the first node is targeted
	if (base == b)
		base = base->next;
	else	// Otherwise it's in the middle or at the end
	for (p = base; p; p = p->next)
		if (p->next == b)
		{
            p->next = b->next;
			break;
		}

	// Free the base's memory
	free(b);
}

/* destroy_mbase():
 *   Destroys a main-base given a pointer to it.
 */
void destroy_mbase(struct MBASE_NODE *b)
{
	struct MBASE_NODE *p;

	if (player.mbase == b)
		player.mbase = NULL;

	// If the first node is targeted
	if (mbase == b)
		mbase = mbase->next;
	else	// Otherwise it's in the middle or at the end
	for (p = mbase; p; p = p->next)
		if (p->next == b)
		{
            p->next = b->next;
			break;
		}

	// Free the base's memory
	free(b);
}