
// comm.h: Communications include file for Bapple2 by Tom Grandgent '97

#ifndef _COMM_

#define _COMM_

#define COMM_NONE	0
#define COMM_CLIENT	1
#define COMM_SERVER	2

#define CONN_NONE	0
#define CONN_WSOCK  1
#define CONN_SERIAL	2

// Initialization states
#define INIT_SEND_IREQ			0		// Send initialization request
#define INIT_RECV_IACK			1		// Receive initialization request acknowledgement
#define INIT_SEND_MHDRREQ   	2		// Send map header request
#define INIT_RECV_MHDR			3		// Receive map header
//#define INIT_SEND_TERRAINREQ 	4		// Send terrain data requests until all received
//#define INIT_SEND_MAPREQ		5		// Send map data requests until all received
#define INIT_SEND_BASEREQ		4		// Send bases request
#define INIT_RECV_BASES			5		// Receive bases
#define INIT_SEND_MBASEREQ		6		// Send mini-bases request
#define INIT_RECV_MBASES		7		// Receive mini-bases
#define INIT_SEND_UNITREQ		8 		// Send units request
#define INIT_RECV_UNITS			9 		// Receive units
#define INIT_SEND_CLREQ			10		// Send clients request
#define INIT_RECV_CLIENTS		11		// Receive clients
#define INIT_START_GAME			12		// Start the actual game play (end initialization)

/////// PACKET TYPES

// Player state update
#define PACKET_PLSTATE	0

// INITIALIZATION PACKETS
/////////////////////////

// Initial connection negotiation
#define PACKET_CREQ		1
#define PACKET_CACK		2

// Final connection negotiation
#define PACKET_IREQ		3
#define PACKET_IACK		4

// Map header request
#define PACKET_MAPHDRREQ	10
// Map header
#define PACKET_MAPHDR		11
// Terrain request
#define PACKET_TERRAINREQ	12
// Terrain data
#define PACKET_TERRAIN		13
// Map request
#define PACKET_MAPREQ		14
// Map data
#define PACKET_MAP			15
// Map row request
//#define PACKET_MAPROWREQ	13
// Map row
//#define PACKET_MAPROW	14
// Base request
#define PACKET_BASEREQ	16
// Base data
#define PACKET_BASE		17
// Main-base request
#define PACKET_MBASEREQ	18
// Main-base data
#define PACKET_MBASE	19
// Unit request
#define PACKET_UNITREQ	20
// Unit data
#define PACKET_UNIT		21
// Client info request
#define PACKET_CLREQ	22
// Client info
#define PACKET_CLIENT	23

// GAME PACKETS
///////////////

// Player hurt signal
#define PACKET_HURT 	40

// Player death signal
#define PACKET_DEATH	41

// Player regeneration signal
#define PACKET_REGEN	42

// Create projectile signal
#define PACKET_PROJ 	43

// Create burst signal
#define PACKET_BURST 	44

// Create unit signal
#define PACKET_NEWUNIT	45

// Unit state packet
#define PACKET_USTATE   46

// Unit kill request
#define PACKET_UKILL	47

// Message packet
#define PACKET_MSG		50

// High and low frequency player state update packet rate requests
#define PACKET_HIGH		51
#define PACKET_LOW		52

// Kill client request
#define PACKET_KILLCL	60

// Keepalive request
#define PACKET_KPALIVE	61

// Ping request
#define PACKET_PING		62
// Pong response
#define PACKET_PONG		63

// Exit game request
#define PACKET_EXIT 	127

// Initialization request packet
typedef struct {
	signed char type;
	char name[30];
	unsigned char id;
} PACKET_IREQ_T;

// Map header packet
typedef struct {
	signed char type;
	char filename[80];
	int width;
	int height;
} PACKET_MHDR_T;

// Data request packet
typedef struct {
	signed char type;
	unsigned short int packetnum;
} PACKET_DATAREQ_T;

// Map row request packet
typedef struct {
	signed char type;
	short row;
} PACKET_MROWREQ_T;

// Hurt request packet
typedef struct {
	signed char type;
	unsigned char victim;
	unsigned char victor;
	short health;
} PACKET_HURT_T;

// Player state update packet
typedef struct {
	signed char type;			// 1 byte
	char x, y;					// 2 bytes
	signed char xoff, yoff;		// 2 bytes
	fixed speed;				// 8 bytes
	unsigned char direction;	// 1 byte
	short health;				// 2 bytes
	unsigned char input;		// 1 byte
	unsigned char id;			// 1 byte
} PACKET;						// 18 bytes total

// Projectile creation packet
typedef struct {
	signed char type;			// 1 byte
	signed char ptype;			// 1 byte
	char x, y;					// 2 bytes
	signed char xoff, yoff;		// 2 bytes
	fixed speed;				// 8 bytes
	unsigned char direction;	// 1 byte
	unsigned char id;			// 1 byte
	signed short utarget;		// 2 bytes
	signed char ptarget;		// 2 bytes
} PACKET_PROJ_T;

#define HIGH_FREQ	75
#define LOW_FREQ	200

#endif // #ifndef _COMM_
