// Bapple2 Global Variables
///////////////////////////
#include <allegro.h>
#include "bapple2.h"
#include "msg.h"
#include "comm.h"
#include "map.h"
#include "units.h"
#include "radar.h"

// ----- Main game variables and operational parameters -----

char game_state = GAME_STATE_PLAY;
char quick_start = 0;
char edit_mode = 0;
char map_editor = 0;
char use_mouse = 0;
char use_big_map = 1;
int num_initial_units = 50;
int num_initial_mainbases = 4;
int num_initial_minibases = 8;
char debug = 0;
char showpath = 0;
long heapused = 0;
char motion_blur = 0;

// ----- Help Text -----

char *helptext = 0;

// ----- Graphics system variables -----

// Rendering method (PAGE_FLIP or DOUBLE_BUFFER)
#ifdef WIN32
int gfx_method = DOUBLE_BUFFER;
#else
int gfx_method = PAGE_FLIP;
#endif
// Allegro graphics mode
int gfx_mode = GFX_MODEX;
// Physical screen dimensions
int PhysicalWidth = 320;
int PhysicalHeight = 240;
// Logical screen dimensions
int LogicalWidth = 320 + 64;
int LogicalHeight = 240 + 64;
// The world coordinates for the first tile drawn of the background
signed int bgx = 1, bgy = 1, bgxoff = 0, bgyoff = 0;
// Variables for keeping track of the screen sliding
int bglagx, bglagy, bglagxoff, bglagyoff;
char slide_enabled = 0;
signed int bgxslide = 0, bgyslide = 0;
// White palette, used for fading involving complete saturation
PALETTE whitepal;
// Bitmap definitions for the visible and hidden drawing pages
BITMAP *VisiblePage = 0, *HiddenPage = 0;
// Allegro graphics driver name and description, displayed at end of program
char gfx_name[80];
char gfx_desc[256];
// Default text color (a little bluish white)
//int TEXT_COLOR = 160;
// RGB Mapping Table (speeds up calls to makecol())
RGB_MAP rgb_table;
// Transparency Mapping Table
COLOR_MAP trans_table;

// ----- Player related variables -----

// Main player structure definition
player_t player;
// The number of active players (currently unused)
int active_players = 1;
// Screen pixel coordinates of the player - constantly reassigned
int centerx, centery;
// ID of the unit that the player is currently carrying
int unit_carried = UNIT_NONEXISTENT;
// Afterburner flag
char afterburner = 0;
// Invincibility flag
int invincible = 0;
// Player starting coordinates (reassigned by main base creation)
int plsx = 20, plsy = 20, plsxoff = 11, plsyoff = 15;  // offs: 7, 11
// Death stuff
long death_start_time = 0;
long last_explosion_time = 0;
int explosion_rate = 2;
// Shooting stuff (some of this is currently unused)
int gun_speed[NUM_GUNS] =  {   7,   6,   8,   1,   9,   3};
int gun_status[NUM_GUNS] = {  ON, OFF, OFF, OFF, OFF, OFF};
long gun_rate[NUM_GUNS] =  {   3,   4,   2,   8,   5,   2};
long last_gun_time[NUM_GUNS];
// Order state (for ordering units)  -1 if no order, else unit ID # on order
signed int order_state = -1;
// Order start time
unsigned long order_time = 0;

// Used for the player update interrupt (NOT CURRENTLY USED)
fixed player_update_direction = 0, player_update_speed = 0;

// ----- Enemy related variables -----

// # of active enemies
char active_enemies = 0;
// Main enemy structure definition
player_t enemy[MAX_PLAYERS];

// ----- Projectile and explosion related variables -----

// Projectile declaration
projectile_struct proj[MAX_PROJECTILES];
// Explosion declaration
explosion_struct explosion[MAX_EXPLOSIONS];
// The number of active projectiles
int active_projectiles = 0;
// # of non-active projectiles.. used to find # of truly active ones
int non;
// The number of active explosions
int active_explosions = 0;
// Coordinate offsets for the explosions (to keep them centered)
// This is necessary because the explosion bitmaps are stored with
// optimal, rather than consistent, bounding boxes.
signed char small_exp_xoff[7] = {0,-2,-3,-2,-1,-1,-1};
signed char small_exp_yoff[7] = {0,-2,-3,-1,-1,0,0};
signed char big_exp_xoff[9] = {0,4,0,3,2,1,0,-2,-2};
signed char big_exp_yoff[9] = {0,3,-2,2,1,2,1,0,1};

// ----- Map related variables -----

// Map dimensions
int MAP_HEIGHT = 100;
int MAP_WIDTH = 100;
// Currently used tile graphics filename
char tilefile[80] = { 0 };
// Tile map definition (for the visible tile #'s)
short *map = 0;		// char *map = 0;
// Frill map definition (a layer placed above the normal visible tiles, for bases and such)
short *frill = 0;
// Map used for terrain types of each tile
signed char *terrain = 0;
// Visibility map
char *vis = 0;
// Map used to determine if there are units at any given location
thing_t **unit_at = 0;
// Terrain bitmap, used for optimizing the mini-map
BITMAP *terrain_bmp = 0;
// Flag for redrawing the terrain bitmap for the mini-map
char redraw_terrain_bmp = 1;
// Map object bitmap, used for making radar work with the mini-map
BITMAP *mapobj_bmp = 0;
// Visible mini-map bitmap - basically terrain_bmp + mapobj_bmp, updated by radar
BITMAP *vismap_bmp = 0;
// Used for drawing walls (the start coordinates)
int penx = 10, peny = 10;
// Current map header
map_header_t current_map;
// Current map filename
char current_map_filename[80];

// ----- Radar related variables -----

// Global radar linked list pointer
radar_t *radar = NULL;  // top radar in the linked list

// ----- Base and mini-base related variables -----

// Linked list pointers for the master [base and mini-base] lists
struct BASE_NODE *base;
struct MBASE_NODE *mbase;

// ----- Unit related variables -----

// Global unit linked list pointer
unit_t *units = NULL;  // top unit in the linked list

// Fresh ID value - this is like an autonumber in Access - just used and incremented
int fresh_id = 0;

// Is there a unit ready to be picked up
char unit_ready = 0;

// The number of active units
int active_units = 0;
// The currently desired terrain- this is set and changed depending on
// what sort of terrain a unit can use.  The initial setting of TERRAIN_LAND
// should not matter (does any code rely on it?)
char desired_terrain = TERRAIN_LAND;
// Unit names
char *unit_name[NUM_UNITS] = { "Truck", "PT Boat", "Tank", "Gunship" };
// The desired terrain types for various units
int unit_terrain[NUM_UNITS] = { TERRAIN_LAND, TERRAIN_WATER, TERRAIN_LAND, TERRAIN_WATER };
// The time it takes to order various units (in milliseconds)
int unit_order_time[NUM_UNITS] = { 4000, 6000, 8000, 7000 };
// The cost for each unit ($$$)
int unit_cost[NUM_UNITS] = { 300, 600, 800, 700 };
// Range that the unit can see
int unit_vis_range[NUM_UNITS] = { 3, 4, 6, 7 };

// ----- GUI related variables -----

// Window list pointer
wnd_t *wnd_list = 0;

// ----- File access variables -----

// The main game datafile
DATAFILE *datafile = 0;
// The ships datafile
DATAFILE *shipgfx = 0;
// The chopper datafile
DATAFILE *choppergfx = 0;
// The half-size graphics datafile
DATAFILE *halfdatafile = 0;
// The terrain datafile
DATAFILE *terraingfx = 0;
// The terrain filename
char terrainfile[80] = {'\0'};
// The frill gfx datafile
DATAFILE *frillgfx = 0;
// The number of tiles in the current tile gfx datafile
int NUM_TILES = 0;
// The number of tiles in the current frill gfx datafile
int NUM_FRILL_TILES = 0;
// The number of ship types
int num_ship_types = 0;

// ----- Talking (messaging) variables -----
char talking = 0;			// Is the player currently talking (typing)?
char cursor = 0;			// X position of the cursor, in characters
char talk_buf[400]; 		// Storage area for what the player is typing

// ----- Sound, music, and input related variables -----

int alt_input = 0;
int sound_on = 1;
int music_on = 0;
int joystick_on = 0;
int digi_vol = 255;
int midi_vol = 255;
MIDI *game_music = NULL;

// ----- Statistic tracking variables -----

// Frames-per-second counter variables
int fps = 0;
int frame_count = 0;
long total_frames = 0;
int frames = 0;

// ----- Virtual timer variables -----

volatile unsigned long ms_time = 0;  // This can be relied upon and will not be reset
volatile unsigned long ms_unit = 0;  // This gets reset periodically for unit timing
volatile unsigned long ms_proj = 0;  // This gets reset periodically for projectile timing
volatile unsigned long ms_fly = 0;   // This gets reset periodically for movement timing
volatile unsigned long ms_accel = 0; // This gets reset periodically for acceleration timing
volatile unsigned long ms_turn = 0;  // This gets reset periodically for turn timing
volatile unsigned long ms_regen = 0; // This gets reset periodically for regeneration
volatile unsigned long ms_send = 0;  // This gets reset periodically for sending
// Used to simulate the old timing system based on clock()
volatile unsigned long simclock = 0;

// ----- Communications variables -----

// Communications mode - client, server, or no communications at all
char comm_mode = COMM_NONE;
// Connection type - serial (incomplete), Winsock UDP/IP, or none
char conn_type = CONN_NONE;
// Initialization state
char init_state = 0;
// Time when the last packet was received (set from ms_time)
unsigned long last_packet_time = 0;
// State stream packet send frequencies for player and enemy
int send_frequency = HIGH_FREQ;
int enemy_send_freq = HIGH_FREQ;
// For keeping track of how much data has been received.. (for terrain & map downloads)
int datarecvd = 0;
int packetsrecvd = 0;
// Map packet size
int MAP_PACKET_SIZE = 1000;

// ----- Stupid backup terrain -----

signed int chunk[6][7] = {
-1,-1,14,25,19,-1,-1,
-1,-1,23,26,18,25,19,
14,25,15,21,26,26,22,
23,26,26,26,26,17,16,
12,24,24,13,21,22,-1,
-1,-1,-1,12,24,16,-1
};

signed int chunk_terrain[6][7] = {
-1,-1, 0, 0, 0,-1,-1,
-1,-1, 0, 2, 2, 0, 0,
 0, 0, 2, 2, 2, 2, 0,
 0, 2, 2, 2, 2, 2, 0,
 0, 0, 0, 2, 2, 0,-1,
-1,-1,-1, 0, 0, 0,-1
};

// Map editor variables

// Tile source window coordinates
////int tilesrcx = 0, tilesrcy = 0;
// Tile-under-mouse world coordinates
int mtilex = 0, mtiley = 0;
// Mouse locking variable
char mlock = 0;
// Flag to hide all windows
char hide_all_windows = 0;
// Flag to display terrain type on top of each tile
char display_terrain_type = 0;
// Flag to display tile grid lines
char display_grid = 0;
